class StringParserIterator:
    """
    It's used as the one and only parameter for constructors (__init__) of AST-classes where parsing a file is needed.
    This class can be used as a python-iterator (e.g. using next on it) to iterate over characters of a string. It must
    be constructed from a string containing text to parse.
    """

    def __init__(self, source_text: str):
        self._index = 0
        self._current_line = source_text
        self._generator = None

    def __iter__(self):
        return self

    def __next__(self):
        current_value = self.peek()
        if current_value is None:
            raise StopIteration
        self.advance()
        return current_value

    def clone(self) -> 'StringParserIterator':
        """
        :return: an independent clone of this iterator, with its own state
        """
        cloned = StringParserIterator(self._current_line)
        cloned._index = self._index
        cloned._generator = self
        return cloned

    def advance(self) -> None:
        """
        Advances the iterator by one character. It doesn't return any value,
        to query for the current value of the iterator use the peek() method
        """
        self._index += 1

    def advance_by(self, n: int) -> None:
        """
        Advances the iterator by n character. It doesn't return any value,
        to query for the current value of the iterator use the peek() method

        :param n: how many times to advance the iterator
        """
        self._index += n

    def peek(self) -> str | None:
        """
        Used to see the current value of the iterator without advancing it.
        :return: a string of length 1 corresponding to the current value of the iterator or None if end-of-file is reached.
        """
        if self._index < len(self._current_line):
            return self._current_line[self._index]
        return None

    def get_index(self) -> int:
        """
        Used to check the current index of the iterator
        :return: the current index of the iterator
        """
        return self._index

    def synchronize_with_source(self) -> None:
        """
        When called on an iterator that has been created using the clone method on another iterator,
        it will update the state of the original iterator to match the state of this iterator. If the
        iterator has not been created as a clone, it does nothing.
        """
        if self._current_line is not None:
            self._generator._current_line = self._current_line
            self._generator._index = self._index


from minigrammar import *


class MathSettings(LanguageSettings):
    pass


rid = MathSettings.get_id_of_rule_assuming_in_same_module


@repeating(rid("Addend"), 1, None, '+', False, False)
class Expression(MathSettings):
    def __repr__(self):
        string = self.elems[0].__repr__()
        for elem in self.elems[1:]:
            string += " + " + repr(elem)
        return " { " + string + " } "


@repeating(rid("Factor"), 1, None, '*', False, False)
class Addend(MathSettings):
    def __repr__(self):
        string = self.elems[0].__repr__()
        for elem in self.elems[1:]:
            string += " * " + repr(elem)
        return " { " + string + " } "


@either([rid("Number"), rid("Variable"), rid("WrappedExpression")])
class Factor(MathSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@regex_pattern(r'^\d+$')
class Number(MathSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@regex_pattern(r'^[a-z]$')
class Variable(MathSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@chain([rid("OpenParen"), rid("Expression"), rid("ClosedParen")])
class WrappedExpression(MathSettings):
    def __repr__(self):
        return " ( " + self.elems[1].__repr__() + " ) "


@exact_match("(")
class OpenParen(MathSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@exact_match(")")
class ClosedParen(MathSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


if __name__ == "__main__":
    iterator = StringParserIterator("x*(a+b)")
    expr = Expression(iterator)
    print(expr)



import sys
from minigrammar import *


class JsonSettings(LanguageSettings):

    @classmethod
    def ignore_characters(cls, char):
        return char == ' ' or char == '\t' or char == '\r'


rid = JsonSettings.get_id_of_rule_assuming_in_same_module


@either([rid("Array"), rid("Value"), rid("Object")])
class Json(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@chain([rid("OpenSquareBracket"), rid("MultipleArrayElems"), rid("ClosedSquareBracket")])
class Array(JsonSettings):
    def __repr__(self):
        return "[" + self.elems[1].__repr__() + "]"


@chain([rid("OpenCurlyBracket"), rid("MultipleObjectFields"), rid("ClosedCurlyBracket")])
class Object(JsonSettings):
    def __repr__(self):
        return "{" + self.elems[1].__repr__() + "}"


@repeating(rid("Json"), None, None, ',', False, False)
class MultipleArrayElems(LanguageSettings):
    def __repr__(self):
        return self.elems.__repr__()[1:-1]


@repeating(rid("ObjectField"), None, None, ',', False, False)
class MultipleObjectFields(LanguageSettings):
    def __repr__(self):
        return self.elems.__repr__()[1:-1]


@chain([rid("StringLiteral"), rid("Colon"), rid("Json")])
class ObjectField(LanguageSettings):
    def __repr__(self):
        return self.elems[0].__repr__() + " : " + self.elems[2].__repr__()


@either([rid("StringLiteral"), rid("FloatingPointLiteral"), rid("IntegerLiteral"), rid("BooleanLiteral"),
         rid("NullLiteral")])
class Value(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@regex_pattern(r'^"(?:[^"\\]|\\.)*"$')
class StringLiteral(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()[1:-1]


@exact_match("null")
class NullLiteral(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@regex_pattern(r'^\d+$')
class IntegerLiteral(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@regex_pattern(r'^\d*\.\d+$')
class FloatingPointLiteral(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@regex_pattern(r'true|false')
class BooleanLiteral(JsonSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@exact_match("[")
class OpenSquareBracket(LanguageSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@exact_match("]")
class ClosedSquareBracket(LanguageSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@exact_match("{")
class OpenCurlyBracket(LanguageSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@exact_match("}")
class ClosedCurlyBracket(LanguageSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


@exact_match(":")
class Colon(LanguageSettings):
    def __repr__(self):
        return self.elems[0].__repr__()


if __name__ == "__main__":
    with open(sys.argv[1], 'r') as file:
        iterator = FileParserIterator(file)
        json = Json(iterator)
        print(json)
